@@ -1,33 +1,46 @@
-import grokcore.component as grok
-
-from zope.interface import providedBy
-from zope.schema import getFields
+from zc.relation.interfaces import ICatalog
 from zope import component
 from zope.event import notify
-from zope.app.intid.interfaces import IIntIds, IIntIdRemovedEvent
-from zope.app.container.interfaces import (IObjectAddedEvent,
-                                           IObjectRemovedEvent)
-from zope.lifecycleevent.interfaces import IObjectModifiedEvent
+from zope.interface import implementer
+from zope.interface import providedBy
+from zope.intid.interfaces import IIntIds
 from zope.lifecycleevent import ObjectModifiedEvent
+from zope.schema import getFields
 
-from zc.relation.interfaces import ICatalog
+from z3c.relationfield.interfaces import IHasIncomingRelations
+from z3c.relationfield.interfaces import IHasOutgoingRelations
+from z3c.relationfield.interfaces import IRelation
+from z3c.relationfield.interfaces import IRelationBrokenEvent
+from z3c.relationfield.interfaces import IRelationList
+from z3c.relationfield.interfaces import IRelationValue
+from z3c.relationfield.interfaces import ITemporaryRelationValue
+
+
+@implementer(IRelationBrokenEvent)
+class RelationBrokenEvent(ObjectModifiedEvent):
+    __doc__ = IRelationBrokenEvent.__doc__
 
-from z3c.relationfield.interfaces import (IHasOutgoingRelations,
-                                          IHasIncomingRelations,
-                                          IRelation, IRelationList,
-                                          IRelationValue,
-                                          ITemporaryRelationValue)
 
-@grok.subscribe(IHasOutgoingRelations, IObjectAddedEvent)
 def addRelations(obj, event):
     """Register relations.
 
     Any relation object on the object will be added.
     """
     for name, relation in _relations(obj):
-         _setRelation(obj, name, relation)
+        _setRelation(obj, name, relation)
+
+
+# zope.intid dispatches a normal event, so we need to check that
+# the object has relations.  This adds a little overhead to every
+# intid registration, which would not be needed if an object event
+# were dispatched in zope.intid.
+def addRelationsEventOnly(event):
+    obj = event.object
+    if not IHasOutgoingRelations.providedBy(obj):
+        return
+    addRelations(obj, event)
+
 
-@grok.subscribe(IHasOutgoingRelations, IObjectRemovedEvent)
 def removeRelations(obj, event):
     """Remove relations.
 
@@ -45,7 +58,7 @@ def removeRelations(obj, event):
                 # The relation value has already been unindexed.
                 pass
 
-@grok.subscribe(IHasOutgoingRelations, IObjectModifiedEvent)
+
 def updateRelations(obj, event):
     """Re-register relations, after they have been changed.
     """
@@ -62,17 +75,18 @@ def updateRelations(obj, event):
         # The object has not been added to the ZODB yet
         return
 
-    # remove previous relations coming from id (now have been overwritten)
-    # have to activate query here with list() before unindexing them so we don't
-    # get errors involving buckets changing size
+    # remove previous relations coming from id (now have been overwritten) have
+    # to activate query here with list() before unindexing them so we don't get
+    # errors involving buckets changing size
     rels = list(catalog.findRelations({'from_id': obj_id}))
     for rel in rels:
-        catalog.unindex(rel)
+        if hasattr(obj, rel.from_attribute):
+            catalog.unindex(rel)
 
     # add new relations
     addRelations(obj, event)
 
-@grok.subscribe(IIntIdRemovedEvent)
+
 def breakRelations(event):
     """Break relations on any object pointing to us.
 
@@ -92,11 +106,12 @@ def breakRelations(event):
     except KeyError:
         # our intid was unregistered already
         return
-    rels = list(catalog.findRelations({'to_id': intids.getId(obj)}))
+    rels = list(catalog.findRelations({'to_id': obj_id}))
     for rel in rels:
         rel.broken(rel.to_path)
         # we also need to update the relations for these objects
-        notify(ObjectModifiedEvent(rel.from_object))
+        notify(RelationBrokenEvent(rel.from_object))
+
 
 def realize_relations(obj):
     """Given an object, convert any temporary relations on it to real ones.
@@ -110,6 +125,7 @@ def realize_relations(obj):
                 # relation list
                 getattr(obj, name)[index] = relation.convert()
 
+
 def _setRelation(obj, name, value):
     """Set a relation on an object.
 
@@ -131,6 +147,7 @@ def _setRelation(obj, name, value):
     catalog = component.getUtility(ICatalog)
     catalog.index_doc(id, value)
 
+
 def _relations(obj):
     """Given an object, return tuples of name, relation value.
 
@@ -140,6 +157,7 @@ def _relations(obj):
         if IRelationValue.providedBy(relation):
             yield name, relation
 
+
 def _potential_relations(obj):
     """Given an object return tuples of name, index, relation value.
 
@@ -160,10 +178,10 @@ def _potential_relations(obj):
                 yield name, None, relation
             if IRelationList.providedBy(field):
                 try:
-                    l = getattr(obj, name)
+                    l_ = getattr(obj, name)
                 except AttributeError:
                     # can't find the relation list on this object
                     continue
-                if l is not None:
-                    for i, relation in enumerate(l):
+                if l_ is not None:
+                    for i, relation in enumerate(l_):
                         yield name, i, relation
